--[[
This addon for WoW will play a music file when you start to dance. The music files have to 
be placed in the "Music" folder of the addon.
]]

-- Save variables, these will get overwritten during load time with the stored value
DanceMusicEnabled = true;
DanceMusicDruidMerge = true;		-- When true all simular durid forms will share a song
DanceMusicWorgenMerge = true;		-- When true worgen human from will share the human song
DanceMusicShowSong = false;			-- Display song names when true
DanceMusicAltDir = false;			-- Alternate Music Directory

local DanceMusicSongAltDir = 'Interface\\DanceMusic\\'

local DanceMusicDebug = false;		-- Display debug messages when true
local DanceMusicPlaying = false;	-- When true a song is playing that was started by DanceMusic
local DanceMusicDefaultSongname = '';
local DanceMusic_RightClick = false;
local DanceMusic_LeftClick = false;
local DanceMusic_RightTurn = false;
local DanceMusic_LeftTurn = false;
local savedname =''
local modelname =''
local startdance=false;
local stealthdance=false;
local combatdance=false;
local DanceOnMount=false;
local NoDanceOnMount=false;
local DanceMusicEnabledTemp = true
local OnTaxi=false

-- Prints messages in debug mode
function DanceMusic_Debug(msg)
  if DanceMusicDebug then
    DEFAULT_CHAT_FRAME:AddMessage(msg);
  end;
end

-- Slash command handler
function DanceMusic_SlashCommandHandler(command)
	if (command == "on") then
		DanceMusicEnabled = true;
		DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_ENABLED);
	elseif (command == "off") then
		DanceMusicEnabled = false;
		DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_DISABLED);
	elseif (command == "druidmerge") then
		if (DanceMusicDruidMerge == false) then
			DanceMusicDruidMerge = true;
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_DRUID_ON);
		else
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_DRUID_OFF);
			DanceMusicDruidMerge = false;
		end;
	elseif (command == "worgenmerge") then
		if (DanceMusicWorgenMerge == false) then
			DanceMusicWorgenMerge = true;
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_WORGEN_ON);
		else
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_WORGEN_OFF);
			DanceMusicWorgenMerge = false;
		end;
	elseif (command == "debug") then
		if (DanceMusicDebug == false) then
			DanceMusicDebug = true;
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_DEBUG_ENABLED);
		else
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_DEBUG_DISABLED);
			DanceMusicDebug = false;
		end;
	elseif (command == "showsong") then
		if (DanceMusicShowSong == false) then
			DanceMusicShowSong = true;
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_SHOWSONG_ON);
		else
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_SHOWSONG_OFF);
			DanceMusicShowSong = false;
		end;
	elseif (command == "altdir") then
		DanceMusicSongDir = DanceMusicSongAltDir
		DanceMusicAltDir = true;
		DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_ALTDIR_ON);
	elseif (command == "defdir") then
		DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_ALTDIR_OFF);
		DanceMusicSongDir = 'Interface\\AddOns\\DanceMusic\\Music\\'
		DanceMusicAltDir = false;
	else
		-- Print current status and help message
		if (DanceMusicEnabled) then
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_ENABLED);
		else
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_DISABLED);
		end;
		if (DanceMusicShowSong == false) then
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_SHOWSONG_OFF);
		else
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_SHOWSONG_ON);
		end;
		if (DanceMusicDruidMerge) then
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_DRUID_ON);
		else
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_DRUID_OFF);
		end;
		if (DanceMusicWorgenMerge) then
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_WORGEN_ON);
		else
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_WORGEN_OFF);
		end;
		if (DanceMusicAltDir) then
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_ALTDIR_ON);
		else
			DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_ALTDIR_OFF);
		end;
		DEFAULT_CHAT_FRAME:AddMessage(DANCEMUSIC_HELP);
	end;
end;

function DanceMusic_OnLoad()
	DanceMusic:RegisterEvent("UNIT_SPELLCAST_SENT");
	DanceMusic:RegisterEvent("UNIT_MODEL_CHANGED");
	DanceMusic:RegisterEvent("UNIT_AURA");
	DanceMusic:RegisterEvent("WORLD_MAP_UPDATE");
	DanceMusic:RegisterEvent("PLAYER_REGEN_ENABLED");
	DanceMusic:RegisterEvent("PLAYER_REGEN_DISABLED");
	DanceMusic:RegisterEvent("PLAYER_GAINS_VEHICLE_DATA");
	DanceMusic:RegisterEvent("PLAYER_LOSES_VEHICLE_DATA");
	DanceMusic:RegisterEvent("PLAYER_ENTER_COMBAT");
	DanceMusic:RegisterEvent("PLAYER_LEAVE_COMBAT");
	DanceMusic:RegisterEvent("PLAYER_CONTROL_GAINED");
	DanceMusic:RegisterEvent("PLAYER_CONTROL_LOST");
	DanceMusic:RegisterEvent("PLAYER_LEAVING_WORLD");
	
	-- Register the slash command
	SlashCmdList["DanceMusic"] = DanceMusic_SlashCommandHandler;
	SLASH_DanceMusic1 = "/dancemusic";

	-- Hook into the DoEmote function to see if we start dancing
	hooksecurefunc("DoEmote", DanceMusic_DoEmote);

	-- Dance stops when moving, so hook relevant movement functions to stop the music. 
	hooksecurefunc("MoveForwardStart"	, DanceMusic_PlayerMove);
	hooksecurefunc("MoveBackwardStart"	, DanceMusic_PlayerMove);
	hooksecurefunc("StrafeLeftStart"	, DanceMusic_PlayerMove);
	hooksecurefunc("StrafeRightStart"	, DanceMusic_PlayerMove);
	hooksecurefunc("JumpOrAscendStart"	, DanceMusic_PlayerMove);
	hooksecurefunc("ToggleAutoRun"		, DanceMusic_PlayerMove);
	
	hooksecurefunc("TurnOrActionStart"			, DanceMusic_TurnOrActionStart);
	hooksecurefunc("TurnOrActionStop"			, DanceMusic_TurnOrActionStop);
	hooksecurefunc("CameraOrSelectOrMoveStart"	, DanceMusic_CameraOrSelectOrMoveStart);
	hooksecurefunc("CameraOrSelectOrMoveStop"	, DanceMusic_CameraOrSelectOrMoveStop);
	hooksecurefunc("TurnLeftStart"				, DanceMusic_TurnLeftStart);
	hooksecurefunc("TurnLeftStop"				, DanceMusic_TurnLeftStop);
	hooksecurefunc("TurnRightStart"				, DanceMusic_TurnRightStart);
	hooksecurefunc("TurnRightStop"				, DanceMusic_TurnRightStop);

	playermodel = CreateFrame("PlayerModel")
		
	DanceMusic_Debug('DanceMusic_OnLoad');
end

function DanceMusic_OnUpdate()
	if ( (Map_Changed) and not WorldMapFrame:IsVisible() ) then
		DanceMusic_StopMusic();
		Map_Changed = false;
	end;
	
	if (UnitIsAFK("player") and (DanceMusicPlaying == true)) then
		DanceMusic_StopMusic();
	end;
	
	if (UnitOnTaxi("palyer") and OnTaxi==false) then
		OnTaxi=true;
	elseif (not UnitOnTaxi("palyer") and OnTaxi==true) then
		OnTaxi=false;
	end;
	
	if (UnitOnTaxi("palyer") and DanceOnMount==true) then
		DanceOnMount=false;
	end;
end

function DanceMusic_OnEvent(self, event, unit, spellName, ...)
	DanceMusic_Debug(event);
	if ( (event == "WORLD_MAP_UPDATE") and WorldMapFrame:IsVisible() ) then
		Map_Changed = true;
	end
	
	if (event=="PLAYER_LEAVING_WORLD" and DanceMusicPlaying==true) then
		DoEmote("DANCE");
	end;
	
	if (event=="UNIT_AURA" and unit=="player" and IsMounted()==nil)then
		if (DanceOnMount==true and NoDanceOnMount==true) then
			DanceOnMount=false;
			NoDanceOnMount=false;
		elseif (DanceOnMount==true and NoDanceOnMount==false) then
			DanceOnMount = false
			DanceMusic_Debug('Start song: '..DanceMusicSongDir..DanceMusicDefaultSongname);
			PlayMusic(DanceMusicSongDir..DanceMusicDefaultSongname);
			DanceMusicPlaying = true;
			if startdance==true then
				DoEmote("DANCE");
				startdance=false;
			end;
		elseif (stealthdance==true) then
			DoEmote("DANCE");
		end;
	end;
	
	if (event=="PLAYER_LOSES_VEHICLE_DATA" or event=="PLAYER_CONTROL_GAINED") then
		DanceMusicEnabledTemp = true;
	end;

	if (event=="PLAYER_GAINS_VEHICLE_DATA" or event=="PLAYER_CONTROL_LOST") then
		DanceMusicEnabledTemp = false;
		DanceMusic_StopMusic();
	end;
	
	if ((event=="PLAYER_REGEN_DISABLED" or event=="PLAYER_ENTER_COMBAT" or event=="PLAYER_REGEN_ENABLED" or event=="PLAYER_LEAVE_COMBAT") and (DanceMusicPlaying == true)) then
		combatdance = true;
		DoEmote("DANCE");
	end;
	
	if (unit == "player" and event=="UNIT_SPELLCAST_SENT" and DanceMusicEnabledTemp==true) then
		if (spellName=="Running Wild")then
			NoDanceOnMount=true;
			DanceMusic_Debug('NoDanceOnMount')
		end;
	end;
	
	if ((unit == "player") and (DanceMusicPlaying == true)) then
		if (event=="UNIT_SPELLCAST_SENT") then
			DanceMusic_Debug(spellName);
			if (spellName=="Two Forms" or spellName=="Bear Form" or spellName=="Cat Form" or spellName=="Travel Form" or spellName=="Moonkin Form" or spellName=="Tree of Life" or spellName=="Ghost Wolf")then
				StopMusic();
				startdance = true;
			elseif (spellName=="Stealth" or spellName=="Shadowmeld" or spellName=="Prowl")then
				stealthdance=true;
			else
				DanceMusic_PlayerMove()
			end;
		end;
		
		if(event=="UNIT_MODEL_CHANGED" and DanceMusicPlaying==true) then
			playermodel:SetUnit("player");
			modelname = playermodel:GetModel();
			DanceMusic_Debug('\n new: '..modelname..'\n old: '..savedname);
			if (modelname ~= savedname) then
				savedname = modelname;
				DanceMusic_GetSong();
				
				if startdance==true then
					DoEmote("DANCE");
					startdance=false;
				end;
				DanceMusic_Debug('Start song: '..DanceMusicSongDir..DanceMusicDefaultSongname);
				PlayMusic(DanceMusicSongDir..DanceMusicDefaultSongname);
				DanceMusicPlaying = true;
			end;
		end;
	end;
end
	
function DanceMusic_GetSong()
	race = UnitRace("player");

	revmodel = strrev(modelname);
	namestart = strlen(modelname) - strfind(revmodel,'\\') +2;
	songname = strsub(modelname,namestart);

	songname = gsub(songname,'scourgefemale','undeadfemale');
	songname = gsub(songname,'scourgemale','undeadmale');

	if (DanceMusicWorgenMerge == false) then
		if (race=='Worgen') then
			songname = gsub(songname,'human','worgenhuman');
		end;
	end;
	
 	if (DanceMusicDruidMerge == true) then 
		if (strmatch(songname,'druidbear') ~= nil) then
			songname = 'druidbear.mp3';
		elseif (strmatch(songname,'druidcat') ~= nil or strmatch(songname,'tiger') ~= nil) then
			songname = 'druidcat.mp3';
		elseif (strmatch(songname,'druidowlbear') ~= nil) then
			songname = 'druidowlbear.mp3';
		end;
	end;

	DanceMusicDefaultSongname = gsub(songname,'.m2','.mp3');
	if (DanceMusicShowSong==true) then
		DEFAULT_CHAT_FRAME:AddMessage(DanceMusicDefaultSongname);
	end;
	
	if (DanceMusicAltDir) then
		DanceMusicSongDir = DanceMusicSongAltDir
	else
		DanceMusicSongDir = 'Interface\\AddOns\\DanceMusic\\Music\\'
	end;	
	
end

function DanceMusic_DoEmote(emote, msg)
	DanceMusic_Debug('DanceMusic_DoEmote '..emote);

	--emote is DANCE when "/dance" is used.
	
	if (DanceMusicEnabledTemp and DanceMusicEnabled and emote=="DANCE" and OnTaxi==false and UnitIsDeadOrGhost("player")==nil and UnitAura("player", "Feign Death")~="Feign Death") then
		-- Play the song if it is not already playing
		if (DanceMusicPlaying==false) then
			DanceOnMount = false;
			playermodel:SetUnit("player");
			modelname = playermodel:GetModel();
			savedname = modelname;
			DanceMusic_GetSong();

			if (IsMounted()~=nil) then
				DanceOnMount = true;
			else
				DanceMusic_Debug('Start song: '..DanceMusicSongDir..DanceMusicDefaultSongname);
				PlayMusic(DanceMusicSongDir..DanceMusicDefaultSongname);
				DanceMusicPlaying = true;
			end;
		elseif (stealthdance==true) then
			stealthdance=false;
		elseif (combatdance==true) then
			combatdance = false;
		else
			DanceMusic_StopMusic();
			DanceMusic_Debug('Song is already playing');
		end;
	end;
end

function DanceMusic_PlayerMove()
	DanceOnMount = false
	DanceMusicLastMove = GetTime();
	DanceMusic_StopMusic();
end

-- Stop the music
function DanceMusic_StopMusic()
	--DanceMusic_Debug('Stop song when playing');

	-- Only stop the music when DanceMusic was playing
	if (DanceMusicPlaying) then
		StopMusic();
		DanceMusicPlaying = false;
		
		DanceMusic_Debug('Song is stopped');
	end;
end;

function DanceMusic_TurnOrActionStart()
	DanceMusic_RightClick = true;
	DanceMusic_MouseMove();
end

function DanceMusic_TurnOrActionStop()
	DanceMusic_RightClick = false;
	DanceMusic_MouseMove();
end

function DanceMusic_CameraOrSelectOrMoveStart()
	DanceMusic_LeftClick = true;
	DanceMusic_MouseMove();
end

function DanceMusic_CameraOrSelectOrMoveStop()
	DanceMusic_LeftClick = false;
	DanceMusic_MouseMove();
end

function DanceMusic_TurnLeftStart()
	DanceMusic_LeftTurn = true;
	DanceMusic_MouseMove();
end

function DanceMusic_TurnLeftStop()
	DanceMusic_LeftTurn = false;
	DanceMusic_MouseMove();
end

function DanceMusic_TurnRightStart()
	DanceMusic_RightTurn = true;
	DanceMusic_MouseMove();
end

function DanceMusic_TurnRightStop()
	DanceMusic_RightTurn = false;
	DanceMusic_MouseMove();
end

function DanceMusic_MouseMove()
	if (DanceMusicPlaying) then
		if (DanceMusic_LeftClick and DanceMusic_RightClick) then
			DanceMusic_StopMusic();
		elseif (DanceMusic_RightClick and DanceMusic_RightTurn) then
			DanceMusic_StopMusic();
		elseif (DanceMusic_RightClick and DanceMusic_LeftTurn) then
			DanceMusic_StopMusic();
		end;
	end;
end